@@ -57,6 +57,10 @@
 #' @param outputFileBasename Name of the output file where the classifier setup results ('Training mode') or the analysis results ('Analysis mode') are to be stored. If not assigned, a filename is automatically generated. File names of other files created by the function are generated by appending suitable labels to the provided "outputFileBasename".
 #' @param sep Field separator character for the output files; the default is tabulation.
 #' @param plotFormat String specifying the format of generated graphic files (plots): can either be "pdf" (default) or "png".
+#' @param scorePlotAscending Boolean option to set the direction in which samples are ordered: TRUE corresponds to samples ordered by ascending score, FALSE corresponds to samples ordered by descending score. Default is TRUE. This argument is meaningful only if saveOutputFile is set to TRUE and the function is running in 'Training mode'.
+#' @param scorePlotParameters String specifying the y-axis parameters of the score plot. If empty, the axis is configured by assessing suitable parameters from the data.  This argument is meaningful only if saveOutputFile is set to TRUE and the function is running in 'Training mode'. The string has to comply with the format "yl_yu_yt", where: yl is the lower y limit; yu is the upper y limit; yt is the interval between tics along the axis.
+#' @param histogramParameters String specifying the parameters used to build histograms. If empty, histograms are built by assessing suitable parameters from the data. This parameter is meaningful only if saveOutputFile is set to TRUE. The string has to comply with the following format: "xl_xu_bw", where xl is the lower boundary of the leftmost bin; xu is the upper boundary of the rightmost bin; bw is the bin width.
+#' @param colorComplementFlag Boolean option to switch between the default palette (FALSE) and its inverted version (TRUE). Default is FALSE, corresponding to target samples reported in blue and versus samples in red. This argument is meaningful only if saveOutputFile is set to TRUE.
 #'
 #' Beware! Cross-correlation coefficients, as well as Shapiro-Wilk tests for normality, require at least three data samples. In case of less than three samples, those tests are skipped and "NA" (not available) is reported in the corresponding output.
 #'
@@ -64,7 +68,7 @@
 #'
 #' @examples
 #' requiredFile = paste(system.file(package="MiRNAQCD"),
-#'			"/extdata/dataset_alpha_clean.dat", sep='')
+#'			"/extdata/test_dataset_alpha_clean.dat", sep='')
 #' myDataFrame <- read.table(file=requiredFile, header=TRUE)
 #' Target <- c("A")
 #' Versus <- c("B", "C")
@@ -80,7 +84,7 @@
 #'					mirnaToUse, coefficientsToUse)
 
 #' @export
-miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList=character(), inputMiRNAList=character(), coeffList=double(), saveOutputFile=FALSE, outputFileBasename="", sep='\t', plotFormat="pdf") {
+miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList=character(), inputMiRNAList=character(), coeffList=double(), saveOutputFile=FALSE, outputFileBasename="", sep='\t', plotFormat="pdf", scorePlotAscending=TRUE, scorePlotParameters=character(), histogramParameters=character(), colorComplementFlag=FALSE) {
 
 	## Input validation and pre-processing
 
@@ -273,7 +277,11 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 			dataFrameTemp[,feature] <- dataFrameTemp[,feature] * as.numeric(coeffList[which(listOfFeature == feature)])
 
 		completeDataFrame <- cbind(completeDataFrame, Score=rowSums(dataFrameTemp))
-		completeDataFrame <- data.frame(completeDataFrame[with(completeDataFrame, order(Score)),])
+		if (scorePlotAscending) {
+			completeDataFrame <- data.frame(completeDataFrame[with(completeDataFrame, order(Score)),])
+		} else {
+			completeDataFrame <- data.frame(completeDataFrame[with(completeDataFrame, order(-1.0*Score)),])
+		}
 
 		## Processing: 2. Means, StdDevs, NormalityTest
 
@@ -371,30 +379,32 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 				return(c(chi,pc))
 			}
 
+			alpha <- 1/sT/sT - 1/sV/sV
+
 			# pv/pt = 1
 			chi_dChi <- compute_chi_dChi(xT, xV, sT, sV, dxT, dxV, dsT, dsV, 1)
 			chi <- chi_dChi[1:2]
 			pc <- chi_dChi[3:4]
 			dchi <- chi_dChi[5:6]
 			dpc <- chi_dChi[7:8]
-			dpc <- dpc[which.min(((chi-(xT + xV)/2)**2))]
-			pc <- pc[which.min(((chi-(xT + xV)/2)**2))]
-			dchi <- dchi[which.min(((chi-(xT + xV)/2)**2))]
-			chi <- chi[which.min(((chi-(xT + xV)/2)**2))]
+			pc <- pc[which.min(alpha*chi)]
+			dpc <- dpc[which.min(alpha*chi)]
+			dchi <- dchi[which.min(alpha*chi)]
+			chi <- chi[which.min(alpha*chi)]
 
 			# pv/pt = likelyhoodRatio: 90% Target threshold
 			chi_dChi <- compute_chi_dChi(xT, xV, sT, sV, dxT, dxV, dsT, dsV, 9.)
 			chiUp <- chi_dChi[1:2]
 			dchiUp <- chi_dChi[5:6]
-			dchiUp <- dchiUp[which.min(((chi-(xT + xV)/2)**2))]
-			chiUp <- chiUp[which.min(((chi-(xT + xV)/2)**2))]
+			dchiUp <- dchiUp[which.min(alpha*chiUp)]
+			chiUp <- chiUp[which.min(alpha*chiUp)]
 
 			# pv/pt = likelyhoodRatio: 90% Versus threshold
 			chi_dChi <- compute_chi_dChi(xT, xV, sT, sV, dxT, dxV, dsT, dsV, 1./9.)
 			chiDown <- chi_dChi[1:2]
 			dchiDown <- chi_dChi[5:6]
-			dchiDown <- dchiDown[which.min(((chi-(xT + xV)/2)**2))]
-			chiDown <- chiDown[which.min(((chi-(xT + xV)/2)**2))]
+			dchiDown <- dchiDown[which.min(alpha*chiDown)]
+			chiDown <- chiDown[which.min(alpha*chiDown)]
 
 			# ROC and corresponding area under curve
 			pred <- completeDataFrame$Score
@@ -402,8 +412,22 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 			pROC_data <- suppressMessages(pROC::roc(obs, pred))
 			ciAuc <- suppressMessages(pROC::ci.auc(pROC_data))
 
-			outputThresholdFrame <- data.frame(chi, dchi, dee, pc, chiUp, dchiUp, chiDown, dchiDown, ciAuc[2], ciAuc[1], ciAuc[3])
-			names(outputThresholdFrame) <- c("Threshold", "DeltaThreshold", "DPrime", "Pc", "ChiUp", "DChiUp", "ChiDown", "DChiDown", "AUC", "AUCDown", "AUCUp")
+			# Performance metrics
+			real_targets <- completeDataFrame[completeDataFrame$Diagnosis == "target", ]
+			real_versus <- completeDataFrame[completeDataFrame$Diagnosis == "versus", ]
+			nr_target_true <- NROW(real_targets[real_targets$Score >= chi, ])
+			nr_target_false <- NROW(real_targets[real_targets$Score < chi, ])
+			nr_versus_true <- NROW(real_versus[real_versus$Score < chi, ])
+			nr_versus_false <- NROW(real_versus[real_versus$Score >= chi, ])
+			performance_sensitivity <- nr_target_true / (nr_target_true + nr_target_false)
+			performance_specificity <- nr_versus_true / (nr_versus_true + nr_versus_false)
+			# performance_accuracy <- (nr_target_true + nr_versus_true) / (nr_target_true + nr_versus_true + nr_target_false + nr_versus_false)
+			performance_accuracy <- pc
+			performance_accuracy_error <- dpc
+			performance_F1score <- 2.0*nr_target_true / (2.0*nr_target_true + nr_target_false + nr_versus_false)
+
+			outputThresholdFrame <- data.frame(chi, dchi, chiUp, dchiUp, chiDown, dchiDown, performance_accuracy, performance_accuracy_error, performance_specificity, performance_sensitivity, performance_F1score, dee, ciAuc[2], ciAuc[1], ciAuc[3])
+			names(outputThresholdFrame) <- c("Threshold", "DeltaThreshold", "ChiUp", "DChiUp", "ChiDown", "DChiDown", "Accuracy", "DAccuracy", "Specificity", "Sensitivity", "F1-score", "DPrime", "AUC", "AUCDown", "AUCUp")
 		}
 
 		## Processing: 5b. In the case of feature analysis (not Classification), prepare output
@@ -426,6 +450,15 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 				outputStatisticsFrame <- rbind(outputStatisticsFrame, rowTarget)
 				outputStatisticsFrame <- rbind(outputStatisticsFrame, rowVersus)
 			}
+		} else {
+			if (!saveOutputFile) {
+				confusionMatrixFileName <- tempfile()
+				cat("Confusion matrix:\tPredicted\tPredicted\n", append=FALSE, file=confusionMatrixFileName)
+				cat("\t\t\tTarget\t\tVersus\n", append=TRUE, file=confusionMatrixFileName)
+				cat(paste("Actual Target\t\t", nr_target_true, "\t\t", nr_target_false, "\n", sep=""), append=TRUE, file=confusionMatrixFileName)
+				cat(paste("Actual Versus\t\t", nr_versus_false, "\t\t", nr_versus_true, "\n", sep=""), append=TRUE, file=confusionMatrixFileName)
+				cat(readLines(confusionMatrixFileName), sep="\n")
+			}
 		}
 
 		## Output: filename building
@@ -451,11 +484,13 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 			if (classifierFlag) {
 				if (outputFileBasename == "") {
 					thresholdFileName <- paste(targetVSVersusLabel, "u", classifierLabel, sep="_")
-					thresholdFileName <- paste(thresholdFileName, "_thresholds", ".txt", sep="")
 					scoresFileName <- paste(thresholdFileName, "_scores", ".dat", sep="")
+					confusionMatrixFileName <- paste(thresholdFileName, "_confusion_matrix", ".txt", sep="")
+					thresholdFileName <- paste(thresholdFileName, "_thresholds", ".txt", sep="")
 				} else {
 	 				thresholdFileName <- paste(outputFileBasename, ".txt", sep="")
 					scoresFileName <- paste(outputFileBasename, ".dat", sep="")
+					confusionMatrixFileName <- paste(outputFileBasename, "_confusion_matrix.txt", sep="")
 	 			}
 
 				if (file.exists(thresholdFileName) & file.access(thresholdFileName, mode=2)) {
@@ -466,19 +501,26 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 					stop("ERROR:\tcannot write ", scoresFileName, ". Check write permission.\n")
 				}
 
+				if (file.exists(confusionMatrixFileName) & file.access(confusionMatrixFileName, mode=2)) {
+					stop("ERROR:\tcannot write ", confusionMatrixFileName, ". Check write permission.\n")
+				}
+
 				cat("#classifier: ", file=thresholdFileName)
 				for (i in 1:length(coeffList)) {
 					cat(paste(coeffList[i], "*", listOfFeature[i], sep=""), " ", file=thresholdFileName, append = TRUE)
 				}
 				cat("\n", file=thresholdFileName, append = TRUE)
 
-
 				utils::write.table(format(outputThresholdFrame, drop0trailing=FALSE), file=thresholdFileName, append=TRUE, sep=sep, row.names=FALSE, col.names=TRUE, quote=FALSE)
 				utils::write.table(format(subset(completeDataFrame, select=c("Subject", "Diagnosis", "Score")), drop0trailing=FALSE), file=scoresFileName, append=TRUE, sep=sep, row.names=FALSE, col.names=TRUE, quote=FALSE)
-
+				cat("Confusion matrix:\tPredicted\tPredicted\n", append=FALSE, file=confusionMatrixFileName)
+				cat("\t\t\tTarget\t\tVersus\n", append=TRUE, file=confusionMatrixFileName)
+				cat(paste("Actual Target\t\t", nr_target_true, "\t\t", nr_target_false, "\n", sep=""), append=TRUE, file=confusionMatrixFileName)
+				cat(paste("Actual Versus\t\t", nr_versus_false, "\t\t", nr_versus_true, "\n", sep=""), append=TRUE, file=confusionMatrixFileName)
 
 				message("LOG:\tDiagnostic thresholds data frame written to ", thresholdFileName, " successfully.\n", sep="")
 				message("LOG:\tScores data frame written to ", scoresFileName, " successfully.\n", sep="")
+				message("LOG:\tConfusion matrix written to ", confusionMatrixFileName, " successfully.\n", sep="")
 			}
 
 			## Output: plot (all cases)
@@ -496,14 +538,14 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 		 		}
 
 				if (classifierFlag) {
-					histPlot <- miRNA_plotHistograms(completeDataFrame, outputThresholdFrame, plotFileName, plotFormat=plotFormat)
+					histPlot <- miRNA_plotHistograms(completeDataFrame, outputThresholdFrame, plotFileName, plotFormat=plotFormat, histogramParameters=histogramParameters, colorComplementFlag=colorComplementFlag)
 					message("LOG:\tHistogram plot saved to ", plotFileName, "_histogram.", plotFormat, " successfully.\n", sep="")
-					thresholdPlot <- miRNA_plotThresholds(completeDataFrame, outputThresholdFrame, plotFileName, plotFormat=plotFormat)
+					thresholdPlot <- miRNA_plotThresholds(completeDataFrame, outputThresholdFrame, plotFileName, plotFormat=plotFormat, scorePlotParameters=scorePlotParameters, colorComplementFlag=colorComplementFlag)
 					message("LOG:\tScore classification plot saved to ", plotFileName, "_score.", plotFormat, " successfully.\n", sep="")
 					rocPlot <- miRNA_plotROC(completeDataFrame, plotFileName, plotFormat=plotFormat)
 					message("LOG:\tROC plot saved to ", plotFileName, "_ROC.", plotFormat, " successfully.\n", sep="")
 				} else {
-					histPlot <- miRNA_plotHistograms(completeDataFrame, outputFileLabel=plotFileName, plotFormat=plotFormat)
+					histPlot <- miRNA_plotHistograms(completeDataFrame, outputFileLabel=plotFileName, plotFormat=plotFormat, histogramParameters=histogramParameters, colorComplementFlag=colorComplementFlag)
 					message("LOG:\tHistogram plot saved to ", plotFileName, "_histogram.", plotFormat, " successfully.\n", sep="")
 				}
 			}
@@ -560,7 +602,7 @@ miRNA_classifierSetup <- function(inputDataset, inputTargetList, inputVersusList
 		}
 
 		if (!saveOutputFile) {
-			statisticsFileName=tempfile()
+			statisticsFileName <- tempfile()
 		}
 
 		cat("\n", append=TRUE, file=statisticsFileName)
